-- Handles the creation of the GUI for HealComm 

local currentZone = select(8,GetInstanceInfo())
--local currentZone = 550  --Used for testing (forcing) a specific zone (Don't forget to change its counterpart in the ResetDropdowns function.
local verGame = tonumber(string.match(GetBuildInfo(), "(%w+)%."))
local verAdd = GetAddOnMetadata("HealComm", "Version")
local eras = {"Classic Era", "The Burning Crusade", "The Wrath of the Lich King", "Cataclysm", "Mists of Pandaria", "Warlords of Draenor", "Legion", "Battle for Azeroth", "Shadowlands"}
local eraOverride = 0

-- Short explanation of raidInfo.  It's basically one long table like eras was above.  However, to make it easy to edit and understand, it's 
--   formatted as a multidimensional array, with each line written as a group of three entries.  The format is:   Game Version, IID, "Raid Name",
--   EACH LINE MUST END WITH A COMMA!!! (Technically, the last line doesn't need to, but it doesn't hurt anything, and prevents mistakes when adding additional raids.)
--   For example, Molten Core was introduced in Game Version 1.x, its IID is 409, and it's not the last line, so it's written as:  1,409,"Molten Core",

-- Some raids (like Naxxramas) were introduced in multiple game versions.  In that case list the raid for each version.  (So, Naxx will show in both the Classic and Wrath eras.)

local raidInfo ={
 	1,249,"Onyxia's Lair",
 	1,409,"Molten Core",
	1,469,"Blackwing Lair",
	1,309,"Zul'Gurub",
	1,509,"Ruins of Ahn'Qiraj",
	1,531,"Ahn'Qiraj Temple",
	1,533,"Naxxramas",
	2,532,"Karazhan",
	2,565,"Gruul's Lair",
	2,544,"Magtheridon's Lair",
	2,548,"Serpentshrine Cavern",
	2,550,"Tempest Keep (The Eye)",
	2,534,"Battle for Mount Hyjal",
	2,564,"Black Temple",
	2,568,"Zul'Aman",
	2,580,"Sunwell Plateau",
	3,533,"Naxxramas",
	3,615,"The Obsidian Sanctum",
	3,624,"Vault of Archavon",
	3,616,"The Eye of Eternity",
	3,603,"Ulduar",
	3,649,"Trial of the Crusader",
	3,249,"Onyxia's Lair",
 	3,631,"Icecrown Citadel",
	3,724,"The Ruby Sanctum",
	4,757,"Baradin Hold",
	4,671,"The Bastion of Twilight",
	4,754,"Throne of the Four Winds",
	4,669,"Blackwing Descent",
	4,720,"Firelands",
	4,967,"Dragon Soul",
	5,1008,"Mogu'shan Vaults",
	5,1009,"Heart of Fear",
	5,996,"Terrace of Endless Spring",
	5,1098,"Throne of Thunder",
	5,1136,"Siege of Orgrimmar",
	6,1228,"Highmaul",
	6,1205,"Blackrock Foundry",
	6,1448,"Hellfire Citadel",
	7,1520,"The Emerald Nightmare",
	7,1530,"The Nighthold",
	7,1648,"Trial of Valor",
	7,1676,"Tomb of Sargeras",
	7,1712,"Antorus, the Burning Throne",
	8,1861,"Uldir",
	8,2070,"Battle of Dazar'alor",
	8,2096,"Crucible of Storms",
	8,2164,"The Eternal Palace",
	8,2217,"Ny'alotha, the Waking City",
	9,2296,"Castle Nathria",
}
local raidNum = #raidInfo/3 -- Determines the number of raids.  If you add a raid, and it doesn't show correctly, then you probably didn't correctly follow the format guidelines.
local raidIndex = 1
local raidMatrix = {}
for i = 1, raidNum do
	raidMatrix[i] = {}
	for j = 1, 3 do
		raidMatrix[i][j] = raidInfo[raidIndex]
		raidIndex = raidIndex + 1
	end
end
local currentEra = eras[verGame]
local currentEraNum = verGame
local currentRaid
local currentBoss

local raidZone = {}
for i=1, raidNum do 
	if raidMatrix[i][2] == currentZone then
		raidZone.text = raidMatrix[i][3]
		eraOverride = raidMatrix[i][1]
	end
end

local bosses = {}
bosses["Onyxia's Lair"] = {"Trash", "Onyxia"}
bosses["Molten Core"] = {"Trash", "Lucifron", "Magmadar", "Gehenas", "Garr", "Baron Geddon", "Shazzrah", "Golemagg", "Sulfuron Harbinger", "Majordomo Executus", "Ragnaros"}
bosses["Blackwing Lair"] = {"Trash", "Razorgore the Untamed", "Vaelstrasz", "Broodlord Lashlayer", "Firemaw", "Ebonroc", "Flamegor", "Chrommagus", "Nefarian"}
bosses["Zul'Gurub"] = {"Trash", "High Priestess Jeklik", "High Priest Venoxis", "High Priestess Mar'li", "Bloodlord Mandokir", "Edge of Madness", "High Priest Thekal", "High Priestess Arlokk", "Jin'do the Hexxer", "Hakkar", "Gahz'ranka"}
bosses["Ruins of Ahn'Qiraj"] = {"Trash", "Kurinnaxx", "General Rajaxx", "Moam", "Buru the Gorger", "Ayamiss the Hunter", "Ossirian the Unscarred"}
bosses["Ahn'Qiraj Temple"] = {"Trash", "The Prophet Skeram", "Bug Trio", "Batleguard Sartura", "Fankriss the Unyielding", "Viscidus", "Princess Huhuran", "Twin Emperors", "Ouro", "C'thun"}
bosses["Naxxramas"] = {"Trash","Anub'Rekhan","Grand Widow Faerlina","Maexxna","Noth the Plaguebringer","Heigan the Unclean","Loatheb","Instructor Razuvious","Gothik the Harvester","The Four Horsemen","Patchwerk","Grobbulus","Gluth","Thaddius","Sapphiron","Kel'Thuzad"}
bosses["Karazhan"] = {"Trash", "Attumen the Huntsman", "Moroes", "Maiden of Virtue", "Opera House", "The Curator", "Terestian Illhoof", "Shade of Aran", "Netherspike", "Chess Event", "Prince Malchezaar", "Nightbane", "Servant Quarters"}
bosses["Gruul's Lair"] = {"Trash", "High King Maulgar", "Gruul the Dragonkiller"}
bosses["Magtheridon's Lair"] = {"Magtheridon"}
bosses["Serpentshrine Cavern"] = {"Trash", "Hydross the Unstable", "The Lurker Below", "Leotheras the Blind", "Fathom-Lord Karathress", "Morogrim Tidewalker", "Lady Vashj"}
bosses["Tempest Keep (The Eye)"] = {"Trash", "Al'ar", "Void Reaver", "High Astromancer Solarian", "Kael'thas Sunstrider"}
bosses["Battle for Mount Hyjal"] = {"Trash", "Rage Winterchill", "Anetheron", "Kaz'rogal", "Azgalor", "Archimonde"}
bosses["Black Temple"] = {"Trash", "High Warlord Naj'entus", "Supremus", "Shade of Akama", "Teron Gorefiend", "Gurtogg Bloodboil", "Reliquary of Souls", "Mother Shahraz", "Illidari Council", "Illidan Stormrage"}
bosses["Zul'Aman"] = {"Trash", "Nalorakk", "Akil'zon", "Jan'alai", "Halazzi", "Hex Lord Malacrass", "Zul'jin"}
bosses["Sunwell Plateau"] = {"Trash", "Kalecgos", "Brutallus", "Felmyst", "Eredar Twins", "M'uru", "Kik'jaeden"}
bosses["The Obsidian Sanctum"] = {"Trash", "Tenebron", "Shadron", "Vesperon", "Sartharion"}
bosses["Vault of Archavon"] = {"Trash", "Archavon the Stone Watcher", "Emalon the Storm Watcher", "Koralon the Flame Watcher", "Toravon the Ice Watcher"}
bosses["The Eye of Eternity"] = {"Trash", "Malygos"}
bosses["Ulduar"] = {"Trash", "Flame Leviathan", "Ignis the Furnace Master", "Razorscale", "XT-002 Deconstructor", "The Assembly of Iron", "Kologarn", "Auriaya", "Hodir", "Thorim", "Freya", "Mimiron", "General Vezax", "Yogg-Saron", "Algalon the Observer"}
bosses["Trial of the Crusader"] = {"Trash", "The Northrend Beasts", "Lord Jaraxxus", "Champions of the Alliance", "Champions of the Horde", "Twin Val'kyr", "Anub'arak"}
bosses["Icecrown Citadel"] = {"Trash", "Lord Marrowgar", "Lady Deathwhisper", "Icecrown Gunship Battle", "Deathbringer Saurfang", "Festergut", "Rotface", "Professor Putricide", "Blood Prince Council", "Blood-Queen Lana'thel", "Valithria Dreamwalker", "Sindragosa", "The Lich King"}
bosses["The Ruby Sanctum"] = {"Trash", "Baltharus the Warborn", "General Zarithrian", "Saviana Ragefire", "Halion"}
bosses["Baradin Hold"] = {"Trash", "Argaloth", "Occu'thar", "Alizabal, Mistress of Hate"}
bosses["The Bastion of Twilight"] = {"Trash", "Halfus Wyrmbreaker", "Theralion and Valiona", "Ascendant Council", "Cho'gall", "Sinestra"}
bosses["Throne of the Four Winds"] = {"Trash", "The Conclave of Wind", "Al'Akir"}
bosses["Blackwing Descent"] = {"Trash", "Omnotron Defense System", "Magmaw", "Atramedes", "Chimaeron", "Maloriak", "Nefarian's End"}
bosses["Firelands"] = {"Trash", "Beth'tilac", "Lord Rhyolith", "Alysrazor", "Shannox", "Baleroc, the Gatekeeper", "Majordomo Staghelm", "Ragnaros"}
bosses["Dragon Soul"] = {"Trash", "Morchok", "Warlord Zon'ozz", "Yor'sahj the Unsleeping", "Hagara the Strombinder", "Ultraxion", "Warmaster Blackhorn", "Spine of Deathwing", "Madness of Deathwing"}
bosses["Mogu'shan Vaults"] = {"Trash", "The Stone Guard", "Feng the Accursed", "Gara'jal the Spiritbinder", "The Spirit Kings", "Elegon", "Will of the Emperor"}
bosses["Heart of Fear"] = {"Trash", "Imperial Vizier Zor'lok", "Blade Lord Ta'yak", "Garalon", "Wind Lord Mel'jarak", "Amber-Shaper Un'sok", "Grand Empress Shek'zeer"}
bosses["Terrace of Endless Spring"] = {"Trash", "Protectors of the Endless", "Tsulong", "Lei Shi", "Sha of Fear"}
bosses["Throne of Thunder"] = {"Trash", "Jin'rokh the Breaker", "Horridon", "Council of Elders", "Tortos", "Megaera", "Ji-Kun", "Durumu the Forgotten", "Primordius", "Dark Animus", "Iron Qon", "Twin Consorts", "Lei Shen", "Ra-den"}
bosses["Siege of Orgrimmar"] = {"Trash", "Immerseus", "The Fallen Protectors", "Norushen", "Sha of Pride", "Galakras", "Iron Juggernaut", "Kor'kron Dark Shaman", "General Nazgrim", "Malkorok", "Spoils of Pandaria", "Thok the Bloodthirsty", "Siegecrafter Blackfuse", "Paragons of the Klaxxi", "Garrosh Hellscream"}
bosses["Highmaul"] = {"Trash", "Kargath Bladefist", "The Butcher", "Brackenspore", "Tectus", "Twin Ogron", "Ko'ragh", "Imperator Mar'gok"}
bosses["Blackrock Foundry"] = {"Trash", "Gruul", "Oregorger", "Blast Furnace", "Hans'gar and Franzok", "Flamebender Ka'graz", "Kromog", "Beastlord Darmac", "Operator Thogar", "Iron Maidens", "Blackhand"}
bosses["Hellfire Citadel"] = {"Trash", "Hellfire Assault", "Iron Reaver", "Kormrok", "Kilrogg Deadeye", "Hellfire High Council", "Gorefiend", "Shadow-Lord Iskar", "Socrethar the Eternal", "Tyrant Velhari", "Fel Lord Zakuun", "Xhul'horac", "Mannoroth", "Archimonde"}
bosses["The Emerald Nightmare"] = {"Trash", "Nythendra", "Il'gynoth, Heart of Corruption", "Elerethe Renferal", "Ursoc", "Dragons of Nightmare", "Cenarius", "Xavius"}
bosses["The Nighthold"] = {"Trash", "Skorpyron", "Chronomatic Anomaly", "Trilliax", "Spellblade Aluriel", "Star Augur Etraeus", "High Botanist Tel'arn", "Krosus", "Tichondrius", "Elisande", "Gul'dan"}
bosses["Trial of Valor"] = {"Trash", "Odyn", "Guarm", "Helya"}
bosses["Tomb of Sargeras"] = {"Trash", "Goroth", "Demonic Inquisition", "Harjatan", "Sisters of the Moon", "Mistress Sassz'ine", "The Desolate Host", "Maiden of Vigilance", "Fallen Avatar", "Kil'jaeden"}
bosses["Antorus, the Burning Throne"] = {"Trash", "Garothi Worldbreaker", "Felhounds of Sargeras", "Antoran High Command", "Portal Keeper Hasabel", "Eonar the Life-Binder", "Imonar the Soulhunter", "Kin'garoth", "Varimathras", "Coven of Shivarra", "Aggramar", "Argus the Unmaker"}
bosses["Uldir"] = {"Trash", "Taloc", "MOTHER", "Fetid Devourer", "Zek'voz", "Vectis", "Zul", "Mythrax the Unraveler", "G'huun"}
bosses["Battle of Dazar'alor"] = {"Trash", "Champion of the Light", "Jadefire Masters", "Grong", "Opulence", "Conclave of the Chosen", "King Rastakhan", "High Tinker Mekkatorque", "Stormwall Blockade", "Lady Jaina Proudmoore"}
bosses["Crucible of Storms"] = {"Trash", "The Restless Cabal", "Uu'nat, Harbinger of the Void"}
bosses["The Eternal Palace"] = {"Trash", "Abyssal Commander Shivara", "Blackwater Behemoth", "Radiance of Azshara", "Lady Ashvane", "Orgozoa", "The Queen's Court", "Za'qul", "Queen Azshara"}
bosses["Ny'alotha, the Waking City"] = {"Trash", "Wrathion", "Maut", "The Prophet Skitra", "Dark Inquisitor Xanesh", "The Hive Mind", "Shad'har the Insatiable", "Drest'agath", "Il'gynoth", "Vexiona", "Ra-den the Despoiled", "Carapace of N'Zoth", "N'Zoth the Corruptor"}
bosses["Castle Nathria"] = {"Trash", "Shriekwing", "Huntsman Altimor", "Sun King's Salvation", "Artificer Xy'mox", "Hungering Destoyer", "Lady Inerva Darkvein", "The Council of Blood", "Sludgefist", "Stone Legion Generals", "Sire Denathrius"}

local function createLabel(frame, text)
	local label = frame:CreateFontString(nil,"BACKGROUND", "GameFontNormalSmall")
	label:SetText(text)
	label:SetWidth(100)
	label:SetHeight(25)
	label:SetTextColor(1,.8,0)
	label:SetFont("Fonts\\FRIZQT__.TTF", 10)
	label:SetShadowColor(0,0,0)
	label:SetShadowOffset(1,-1)
	return label
end

local function createEditbox(parent, width, height, tabPressColumn, tabIndex, anchorWidget, xOffset, yOffset)
	local editbox = CreateFrame("EditBox", nil, parent, "InputBoxTemplate")
	editbox:SetWidth(width)
	editbox:SetHeight(height)
	editbox:SetPoint("BOTTOM",anchorWidget,"BOTTOM",xOffset,yOffset)
	editbox:SetAutoFocus(false)
	editbox:SetScript("OnTabPressed", 
		function() 
			if IsShiftKeyDown() and tabPressColumn[tabIndex-1] then
				tabPressColumn[tabIndex-1]:SetFocus()
			elseif tabPressColumn[tabIndex+1] then 
				tabPressColumn[tabIndex+1]:SetFocus()
			end
		end)
	editbox:SetScript("OnEscapePressed", function() HealComm.frame:Hide() end )
	return editbox
end

local function createButton(parent, width, height, text)
	local button = CreateFrame("Button", nil, parent, "UIPanelButtonTemplate")
	button:SetWidth(width)
	button:SetHeight(height)
	button:SetText(text)
	return button
end

function ordinal_numbers(n)
  local ordinal, digit = {"st", "nd", "rd"}, string.sub(n, -1)
  if tonumber(digit) > 0 and tonumber(digit) <= 3 and string.sub(n,-2) ~= 11 and string.sub(n,-2) ~= 12 and string.sub(n,-2) ~= 13 then
    return n .. ordinal[tonumber(digit)]
  else
    return n .. "th"
  end
end

function HealComm:CreateFrame()
	local f = CreateFrame("Frame", "HealCommConfigFrame", UIParent, BackdropTemplateMixin and "BackdropTemplate")
	f:SetFrameStrata("HIGH")
	f:SetWidth(850) 
	f:SetHeight(750)
	f:SetMovable(true)
	f:RegisterForDrag("LeftButton")
	f:EnableMouse(true)
	f:SetScript("OnDragStart", function(self) self:StartMoving() end)
	f:SetScript("OnDragStop", function(self) self:StopMovingOrSizing() end)
	f:SetPoint("CENTER", UIParent)
    f:SetClampedToScreen(true)
    f:SetBackdrop({
    	bgFile="Interface\\DialogFrame\\UI-DialogBox-Background", 
      	edgeFile="Interface\\DialogFrame\\UI-DialogBox-Border", 
      	tile=1, tileSize=16, edgeSize=16, 
      	insets={left=5, right=5, top=5, bottom=5}
	})
	
	-- make the frame referencable by the HC table 
	HealComm.frame = f

	-- insert into UISpecialFrames so that escape key will close the frame 
	tinsert(UISpecialFrames,"HealCommConfigFrame") 

	local title = f:CreateFontString(nil, "OVERLAY", "GameFontNormalLarge")
	title:SetText("HealComm (v"..verAdd..")")
	title:SetWidth(250)
	title:SetHeight(35)
	title:SetPoint("TOPRIGHT",f,"TOPRIGHT", 25, -5)
	title:SetTextColor(1,.6,0)
	title:SetFont("Fonts\\FRIZQT__.TTF", 18)
	title:SetShadowColor(0,0,0)
	title:SetShadowOffset(1,-1)

	-- DROPDOWNS
	-- Era name dropdown 
	HealComm.frame.eraDropdown = CreateFrame("Frame", nil, f, "UIDropDownMenuTemplate")
	UIDropDownMenu_SetWidth(HealComm.frame.eraDropdown, 160)
	HealComm.frame.eraDropdown:SetPoint("TOP", f, "TOP", -295, -75)
	UIDropDownMenu_Initialize(HealComm.frame.eraDropdown, HealComm.EraDropdownOnload)
	if eraOverride > 0 then
		UIDropDownMenu_SetSelectedValue(HealComm.frame.eraDropdown, eras[eraOverride])
		currentEra = eras[eraOverride]
		currentEraNum = eraOverride
	else
		UIDropDownMenu_SetSelectedValue(HealComm.frame.eraDropdown, eras[verGame])
	end

	-- Raid name dropdown 
	HealComm.frame.raidDropdown = CreateFrame("Frame", nil, f, "UIDropDownMenuTemplate")
	UIDropDownMenu_SetWidth(HealComm.frame.raidDropdown, 175)
	HealComm.frame.raidDropdown:SetPoint("TOP", f, "TOP", -100, -75)
	UIDropDownMenu_Initialize(HealComm.frame.raidDropdown, HealComm.RaidDropdownOnload)
	if raidZone.text then 
		UIDropDownMenu_SetSelectedValue(HealComm.frame.raidDropdown, raidZone.text)
		currentRaid = raidZone.text
	end

	-- Boss name dropdown 
	HealComm.frame.bossDropdown = CreateFrame("Frame", nil, f, "UIDropDownMenuTemplate")
	UIDropDownMenu_SetWidth(HealComm.frame.bossDropdown, 175)
	HealComm.frame.bossDropdown:SetPoint("TOP", f, "TOP", 100, -75)
	UIDropDownMenu_Initialize(HealComm.frame.bossDropdown, HealComm.BossDropdownOnload)

	-- DROPDOWN LABELS 
	-- Era name label 
	local eraNameLabel = createLabel(f, "WoW Era")
	eraNameLabel:SetPoint("CENTER",HealComm.frame.eraDropdown,"TOP", 0, 10)

	-- Raid name label 
	local raidNameLabel = createLabel(f, "Raid Name")
	raidNameLabel:SetPoint("CENTER",HealComm.frame.raidDropdown,"TOP", 0, 10)

	-- Boss name label 
	local bossNameLabel = createLabel(f, "Boss Name")
	bossNameLabel:SetPoint("CENTER",HealComm.frame.bossDropdown,"TOP", 0, 10)

	-- HEALER LABEL 
	local healerNameLabel = createLabel(f, "Healer Name")
	healerNameLabel:SetPoint("TOPLEFT",f,"TOPLEFT", 75, -130)

	-- HEALER NAMES AND CLASS ICONS
	HealComm.frame.healerNames = {}
	HealComm.frame.healerIcons = {}
	for i=1, 15 do
		local healerEditbox = createEditbox(f, 150, 25, HealComm.frame.healerNames, i, healerNameLabel, 0, -i*25)
		healerEditbox:SetScript("OnTextChanged", function(self) HealComm:HealerNameChanged(i) end)
		HealComm.frame.healerNames[i] = healerEditbox

		local icon = CreateFrame("Frame", nil, f)
		icon:SetWidth(24)
		icon:SetHeight(24)
		icon:SetPoint("RIGHT",healerEditbox,"LEFT",-5,0)
		icon:SetMovable(true)
		icon:RegisterForDrag("LeftButton")
		icon:EnableMouse(true)
		icon:SetScript("OnDragStart", function(self) HealComm:IconDragStart(self) end)
		icon:SetScript("OnDragStop", function(self) HealComm:IconDragStop(self, i) end)
		icon.texture = icon:CreateTexture()
		HealComm.frame.healerIcons[i] = icon
		icon:Hide()
	end


	HealComm.frame.assignments = {}
	--HealComm.frame.phaseNames = {}
	for i=1, 3 do 
		-- PHASE LABELS
		local phaseLabel = createLabel(f, "Phase "..i.." /")
		phaseLabel:SetPoint("LEFT",healerNameLabel,"RIGHT",-70+i*145,15)
	
		-- ASSIGNMENT LABELS
		local assignmentLabel = createLabel(f, ordinal_numbers(i).." Assignment")
		assignmentLabel:SetPoint("LEFT",healerNameLabel,"RIGHT",-70+i*145,0)


		HealComm.frame.assignments["phase"..i] = {}
		-- ASSIGNMENT EDITBOXES 
		for j=1, 15 do
			local assignmentEditbox = createEditbox(f, 120, 25, HealComm.frame.assignments["phase"..i], j, assignmentLabel, 0, -j*25)
			HealComm.frame.assignments["phase"..i][j] = assignmentEditbox
		end
	end 

	HealComm.frame.notes = {}
	local noteLabel = createLabel(f, "Broadcast Notes")
		noteLabel:SetPoint("TOPLEFT",f,"TOPLEFT", 50, -530)
	local noteEditbox = createEditbox(f, 600, 25, HealComm.frame.noteText, 1, noteLabel, 250, -20)
		noteEditbox:SetText("Everyone else: Raidheal")
		HealComm.frame.notes = noteEditbox

	
	-- PRE-SET ASSIGNMENTS GROUP
	-- heading label  
	local presetAssignmentHeadingLabel = createLabel(f, "Preset Assignments")
	presetAssignmentHeadingLabel:SetWidth(150)
	presetAssignmentHeadingLabel:SetPoint("LEFT",healerNameLabel,"RIGHT",485,0)
	HealComm.frame.presetAssignmentHeadingLabel = presetAssignmentHeadingLabel

	-- dynamic labels: name resolved %MT1-5 (if assigned) (so roster update?)
	-- draggable labels 
	HealComm.frame.presetAssignments = {}
	local staticLabels = {"Raid", "Melee", "Ranged", "Dispel/Decurse", "%MT1", "%MT2", "%MT3", "%MT4", "%MT5"}
	for k, v in pairs (staticLabels) do 
		HealComm:CreatePresetAssignment(v, k)
	end 

	-- BUTTONS
	-- Current Instance button lines
	local cLine = f:CreateLine()
	cLine:SetColorTexture(1,.8,0,.4)
	cLine:SetStartPoint("TOPLEFT",125,-35)
	cLine:SetEndPoint("TOPLEFT",325,-35)
	cLine = f:CreateLine()
	cLine:SetColorTexture(1,.8,0,.4)
	cLine:SetStartPoint("TOPLEFT",125,-35)
	cLine:SetEndPoint("TOPLEFT",125,-50)
	cLine = f:CreateLine()
	cLine:SetColorTexture(1,.8,0,.4)
	cLine:SetStartPoint("TOPLEFT",325,-35)
	cLine:SetEndPoint("TOPLEFT",325,-50)
	
	-- Current Instance button
	local ciButton = createButton(f, 130, 25, "Set Current Instance")
	ciButton:SetPoint("TOP", f, "TOP", -200, -25)
	ciButton:SetScript("OnClick", function(self) HealComm:ResetDropdowns() end)

	
	-- Close button 
	local closeButton = createButton(f, 100, 25, "Close")
	closeButton:SetPoint("BOTTOMRIGHT", f, "BOTTOMRIGHT", -10, 15)
	closeButton:SetScript("OnClick", function(self) self:GetParent():Hide() end)

	-- Broadcast button
	local broadcastButton = createButton(f, 100, 25, "Broadcast")
	broadcastButton:SetPoint("RIGHT", closeButton, "LEFT", -10, 0)
	broadcastButton:SetScript("OnClick", function(self) HealComm:Broadcast() end)

	-- Save button
	local saveButton = createButton(f, 75, 25, "Save")
	saveButton:SetPoint("LEFT", HealComm.frame.bossDropdown, "RIGHT", 5, 2)
	saveButton:SetScript("OnClick", function(self) HealComm:SaveAssignments() end)
	
	-- Clear button
	local clearButton = createButton(f, 75, 25, "Clear")
	clearButton:SetPoint("LEFT", saveButton, "RIGHT", 5, 0)
	clearButton:SetScript("OnClick", function(self) HealComm:ClearAssignments() end)
	
	-- Auto fill button
	local autofillButton = createButton(f, 100, 25, "Auto Fill")
	autofillButton:SetPoint("TOP", HealComm.frame.healerNames[15], "BOTTOM", -25, -80)
	autofillButton:SetScript("OnClick", function(self) HealComm:AutoFillHealers() end)

	-- CHANNEL SETTINGS 
	-- Broadcast to Raid checkbox 
	local broadcastToRaidCheckbox = CreateFrame("CheckButton", "HealComm_broadcastToRaidCheckbox",f, "UICheckButtonTemplate")
	broadcastToRaidCheckbox:SetPoint("BOTTOM", f, "BOTTOM", -50, 140)
	broadcastToRaidCheckbox:SetHeight(25)
	broadcastToRaidCheckbox:SetWidth(25)
	local broadcastToRaidLabel = createLabel(f, "Broadcast to Raid")
	broadcastToRaidLabel:SetPoint("LEFT",broadcastToRaidCheckbox,"RIGHT",-5,0)
	broadcastToRaidCheckbox:SetScript("OnClick", function() HealComm:SaveReportingSettings() end)
	broadcastToRaidCheckbox.tooltip = "Broadcast assignments to raid channel"
	HealComm.frame.broadcastToRaidCheckbox = broadcastToRaidCheckbox

	-- Whisper assignments checkbox 
	local whisperAssignmentsCheckbox = CreateFrame("CheckButton", "HealComm_whisperAssignmentsCheckbox",f, "UICheckButtonTemplate")
	whisperAssignmentsCheckbox:SetPoint("BOTTOM", f, "BOTTOM", -50, 120)
	whisperAssignmentsCheckbox:SetHeight(25)
	whisperAssignmentsCheckbox:SetWidth(25)
	local whisperAssignmentsLabel = createLabel(f, "Whisper Assignments")
	whisperAssignmentsLabel:SetWidth(123)
	whisperAssignmentsLabel:SetPoint("LEFT",whisperAssignmentsCheckbox,"RIGHT",-5,0)
	whisperAssignmentsCheckbox:SetScript("OnClick", function() HealComm:SaveReportingSettings() end)
	whisperAssignmentsCheckbox.tooltip = "Broadcast assignments to raid channel"
	HealComm.frame.whisperAssignmentsCheckbox = whisperAssignmentsCheckbox

	-- Broadcast to custom channel checkbox 
	local broadcastToCustomChannelCheckbox = CreateFrame("CheckButton", "HealComm_broadcastToCustomChannelCheckbox",f, "UICheckButtonTemplate")
	broadcastToCustomChannelCheckbox:SetPoint("BOTTOM", f, "BOTTOM", -50, 100)
	broadcastToCustomChannelCheckbox:SetHeight(25)
	broadcastToCustomChannelCheckbox:SetWidth(25)
	local broadcastToCustomChannelLabel = createLabel(f, "Broadcast to Custom Channel:")
	broadcastToCustomChannelLabel:SetWidth(162)
	broadcastToCustomChannelLabel:SetPoint("LEFT",broadcastToCustomChannelCheckbox,"RIGHT",-5,0)
	broadcastToCustomChannelCheckbox:SetScript("OnClick", function() HealComm:SaveReportingSettings() end)
	broadcastToCustomChannelCheckbox.tooltip = "Broadcast assignments to a custom channel"
	HealComm.frame.broadcastToCustomChannelCheckbox = broadcastToCustomChannelCheckbox

	-- Custom channel name editbox 
	local customChannelEditbox = CreateFrame("EditBox", nil, f, "InputBoxTemplate")
	customChannelEditbox:SetWidth(100)
	customChannelEditbox:SetHeight(25)
	customChannelEditbox:SetPoint("LEFT",broadcastToCustomChannelLabel,"RIGHT",5,0)
	customChannelEditbox:SetAutoFocus(false)
	customChannelEditbox:SetScript("OnEscapePressed", function() HealComm.frame:Hide() end )
	customChannelEditbox:SetScript("OnTextChanged", function() HealComm:SaveReportingSettings() end)
	HealComm.frame.customChannelName = customChannelEditbox

	-- Exception List editbox 
	local exceptionLabel = createLabel(f, "Names to be ignored by Autofill (space-separated)")
	exceptionLabel:SetPoint("TOPLEFT",f,"TOPLEFT", 35, -650)
	exceptionLabel:SetWidth(300)
	local exceptionEditbox = CreateFrame("EditBox", nil, f, "InputBoxTemplate")
	exceptionEditbox:SetWidth(525)
	exceptionEditbox:SetHeight(25)
	exceptionEditbox:SetPoint("TOPLEFT",broadcastToCustomChannelCheckbox,"BOTTOM",-250,-20)
	exceptionEditbox:SetAutoFocus(false)
	exceptionEditbox:SetScript("OnEscapePressed", function() HealComm.frame:Hide() end )
	exceptionEditbox:SetScript("OnTextChanged", function() HealComm:SaveReportingSettings() end)
	HealComm.frame.exceptionList = exceptionEditbox
	-- Apply Exceptions button
	local saveExceptionButton = createButton(f, 75, 25, "Apply")
	saveExceptionButton:SetPoint("TOPRIGHT", exceptionEditbox, "TOPLEFT", -3, 0)
	saveExceptionButton:SetScript("OnClick", function(self) HealComm:UpdateExceptions() end)
end

function HealComm:EraDropdownOnload(self)
	local info = {}
	for i=1,verGame do 
		info.text = eras[i]
		info.value = eras[i]
		info.func = function() 
			UIDropDownMenu_SetText(HealComm.frame.eraDropdown, eras[i]) 
			currentEra = eras[i]
			currentEraNum = i 
			UIDropDownMenu_ClearAll(HealComm.frame.raidDropdown)
			UIDropDownMenu_ClearAll(HealComm.frame.bossDropdown)			
		end 
		UIDropDownMenu_AddButton(info)
	end 
end

-- todo: Find more efficient way to determine Era 
function HealComm:RaidDropdownOnload(self)
	local info = {}
	for i=1, raidNum do 
		if currentEraNum == raidMatrix[i][1] then
			info.text = raidMatrix[i][3]
			info.value = raidMatrix[i][3]
			info.func = function() 
				UIDropDownMenu_SetText(HealComm.frame.raidDropdown, raidMatrix[i][3]) 
				currentRaid = raidMatrix[i][3] 
				UIDropDownMenu_ClearAll(HealComm.frame.bossDropdown)
			end
		UIDropDownMenu_AddButton(info)
		end
	end
end

function HealComm:BossDropdownOnload(self)
	local info = {}
	if not currentRaid then 
		return;
	end
	for i=1, #bosses[currentRaid] do 
		info.text = bosses[currentRaid][i]
		info.value = bosses[currentRaid][i]
		info.func = function() 
			UIDropDownMenu_SetText(HealComm.frame.bossDropdown, bosses[currentRaid][i]) 
			currentBoss = bosses[currentRaid][i]
			HealComm:LoadAssignments()
		end 
		UIDropDownMenu_AddButton(info)
	end 
end

function HealComm:GetCurrentlySelectedRaidAndBoss()
	if currentRaid and currentBoss then 
		return currentRaid.." - "..currentBoss
	else 
		return nil
	end
end

function HealComm:CreatePresetAssignment(text, assignmentIndex)
	local presetAssignment = CreateFrame("Frame", nil, HealComm.frame, BackdropTemplateMixin and "BackdropTemplate")
	presetAssignment:SetWidth(120)
	presetAssignment:SetHeight(25)
    presetAssignment:SetBackdrop( { 
        bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
        edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
        tile = true, tileSize = 16, edgeSize = 16, 
        insets = { left = 4, right = 4, top = 3, bottom = 3 }
    })
    presetAssignment:SetBackdropColor(0, 0, 0, 0.9)
    presetAssignment:SetPoint("TOP", HealComm.frame.presetAssignmentHeadingLabel, "TOP", 0, -25*assignmentIndex)
    presetAssignment.text = createLabel(presetAssignment, text)
    presetAssignment.text:SetPoint("CENTER", presetAssignment, "CENTER")
    presetAssignment.text:SetTextColor(1,1,1)
	presetAssignment:SetMovable(true)
	presetAssignment:RegisterForDrag("LeftButton")
	presetAssignment:EnableMouse(true)
	presetAssignment:SetScript("OnDragStart", function(self) self:StartMoving() end)
	presetAssignment:SetScript("OnDragStop", function(self) HealComm:PresetAssignmentDragStop(self, assignmentIndex) end)
	HealComm.frame.presetAssignments[assignmentIndex] = presetAssignment

end

function HealComm:ResetDropdowns()
	--todo:  Find a way to do this without repeating a whole lot of code.
	currentZone = select(8,GetInstanceInfo())
	--currentZone = 550  --Used for testing specific zone (Don't forget to change its counterpart at the beginning of this file.
	eraOverride = 0
	raidZone.txt = nil
	currentEra = eras[verGame]
	currentEraNum = verGame
	currentRaid = nil
	currentBoss = nil
	UIDropDownMenu_ClearAll(HealComm.frame.raidDropdown)
	UIDropDownMenu_ClearAll(HealComm.frame.bossDropdown)
	raidZone = {}
	for i=1, raidNum do 
		if raidMatrix[i][2]==currentZone then
			raidZone.text = raidMatrix[i][3]
			eraOverride = raidMatrix[i][1]
		end
	end
	UIDropDownMenu_Initialize(HealComm.frame.eraDropdown, HealComm.EraDropdownOnload)
	if eraOverride > 0 then
		UIDropDownMenu_SetSelectedValue(HealComm.frame.eraDropdown, eras[eraOverride])
		currentEra = eras[eraOverride]
		currentEraNum = eraOverride
	else
		UIDropDownMenu_SetSelectedValue(HealComm.frame.eraDropdown, eras[verGame])
	end
	UIDropDownMenu_Initialize(HealComm.frame.raidDropdown, HealComm.RaidDropdownOnload)
	if raidZone.text then 
		UIDropDownMenu_SetSelectedValue(HealComm.frame.raidDropdown, raidZone.text)
		currentRaid = raidZone.text
	end
	UIDropDownMenu_Initialize(HealComm.frame.bossDropdown, HealComm.BossDropdownOnload)
	  	
end
